// ***************************************************************************
//  File Name    : main.c
//  Version      : 1.0
//  Description  : Automated irrigation system 
//  Author       : Per
//  Target       : Microchip PIC16F886 Microcontroller
//  Compiler     : HI-TECH C Compiler for PIC10/12/16 MCUs (Lite Mode)  V9.83
//  IDE          : Microchip MPLAB IDE v8.92
//  Programmer   : PICKit2
//  Last Updated : 2016-05-04
// ***************************************************************************


#include "main.h"
#include <htc.h>

// PIC16F886 Configuration Bit Settings
__CONFIG(FOSC_INTRC_NOCLKOUT & WDTE_OFF & PWRTE_OFF & MCLRE_OFF & CP_OFF & CPD_OFF & BOREN_ON & IESO_ON & FCMEN_OFF & LVP_OFF);
__CONFIG(BOR4V_BOR21V & WRT_OFF);


#define CS RB1
#define WATER 1
#define SUN 0
#define SAMPLES 50
#define OUT 0
#define IN 1
#define NOOFPOTS 20


bank1 unsigned char cmd[50];
unsigned char* cmdP;

char charReceived=0;
unsigned char cmdReceived=0;
unsigned char sunsensor[SAMPLES];
unsigned char timer20s=0;
unsigned char timer1min=0;
unsigned char timer1h=12;
unsigned char hours=12;
unsigned char days=0;
unsigned char sunIndex=0;
unsigned char waterPlants=0;
unsigned char timeToWaterCounter=0;
unsigned short int waterTime;
unsigned short int temp=0;

unsigned char tmp=0;
unsigned short int i=0;
unsigned char open=0;


main(){
	unsigned char suntmp=0;
	unsigned char maxsun=0;
	unsigned char watertmp=0;
	for (tmp=0;tmp++;tmp<49){
		cmd[tmp]='a';
    }
    cmd[49]='\0';

    ANSEL=0b00001100;				//disable analog inputs, pins should be digital i/o except for AN2 and AN3 which is sun- and water sensor
    ANSELH=0b00000000;				//disable analog inputs, except for AN9, which should be used
    CM1CON0=0;						//disable comparators, pins should be digital i/o
    CM2CON0=0;						//disable comparators, pins should be digital i/o

    //TRISA<7:0>: PORTA Tri-State Control bit; 1 = PORTA pin configured as an input (tri-stated); 0 = PORTA pin configured as an output
    TRISA=0b11111111;				//set port a to all intputs, ra4, ra5 output to h-bridge
    TRISB=0b11111111;				//set port b to all intputs
    TRISC=0b11111111;				//set port c to all inputs, RC3 output for transistor;
	
	//Use LFINTOSC
	IRCF2=0;
	IRCF1=0;
	IRCF0=0;
    RC3=0;								
    GIE=1;							//global interrupt enable
    PEIE=1;							//peripheral interrupt enable
    initEusart();					//init eusart module for serial com to PC
    initAd();						//A/D on pin ra1 and ra2

    cmdP=cmd;						//set pointer to start of serial receive buffer (not used)
	
    printf("\n\r");
    printf("=====================================================\n\r");
    printf("=              PIC16F886 bevattning2                =\n\r");
    printf("=====================================================\n\r");
	

	
    printf("\n\r");			

    //init uplwu
	TRISA0=0; //rb0 is digital output
    
    while(1==1){ 
        
		
		////UNCOMMENT BEFORE DEPLYMENT!!!!!
		printf(":");
		
		//Read solar panel voltage, 3,3V=51 5V=80 12V=192
		suntmp=sun();
		
		//if there has been at least 15h since last watering AND its dark outside
		if((timer1h >=15) && (suntmp < 1)){
			printf("Time to water the plants (Sun()<1(actual: %d) and timer1h>=15(actual: %d))!\n\r",suntmp,timer1h);
			printf("Max sun since last watering has been %d\n\r",maxsun);
			printf("Day no. is %d and time is %d\n\r",days,hours);
			
			//Read moisture sensor
			watertmp=water();
			
			//If sensor is dry
			if(watertmp >=250){
				printf("Water sensor not wet (water()>=250(actual: %d))! Watering!\n\r", watertmp);
				
				//open valve
				TRISC3=0;
				TRISA5=0;
				TRISA4=0;
				RC3=1;  			//power to L293E
				delay10ms(10);
				printf("Opening...\n\r");
				RA4=0;  			//opens valve
				RA5=1;  			//opens valve
				delay10ms(800);
				TRISC3=1;
				TRISA5=1;  			//Unnecessary??
				TRISA4=1;  
				RC3=0;  			//no power to L293E
				for(i=0;i<300;i++){
					
					////UNCOMMENT BEFORE DEPLYMENT!!!!!
					delay10ms(100);  //wait 1s
				}
				
				//close the valve...
				TRISC3=0;
				TRISA5=0;
				TRISA4=0;
				RC3=1;  			//power to L293E
				delay10ms(10);
				printf("Closing...\n\r");
				RA4=1;  			//closes valve
				RA5=0;  			//closes valve
				delay10ms(800);
				TRISC3=1;
				TRISA5=1;
				TRISA4=1;
				RC3=0;  			//no power to L293E2
			}else{
			    printf("Water sensor wet (water()<250 (actual: %d), no watering today!\n\r", watertmp);
			}	
			
			//Write an entry in the log (EEPROM)
			log(days, hours, maxsun, watertmp);
			
			//reset maxsun after it has been logged...
			maxsun=0;	
			
			//reset timer after watering 
			timer1h=0;
		}	
		
		//Prep for sleep
		RC3=0;  	//0= not power to L293E
		RA4=0;
		RA5=0;
		TRISA5=1;
		TRISA4=1;
		//printf("Nu sover PICen\n\r");
		TMR1ON=0;
		TMR1H=0;
		TMR1L=0;
		TMR1ON=1;
		asm("CLRWDT");
		//WDTPS3=1;
		//WDTPS2=0;
		WDTPS1=1;
		WDTPS0=1;
		SWDTEN=1;
		
		////UNCOMMENT BEFORE DEPLYMENT!!!!!
		asm("sleep");
		
		
		SWDTEN=0;
		TMR1ON=0;
		//printf("Nu vaknar PICen, tid %02d:%02d:%02d\n\r",timer1h,timer1min,timer20s*20);
		timer20s++;
		if (timer20s==3){
			timer1min++;
			timer20s=0;
			
			//Heartbeat, maybe remove
			////UNCOMMENT BEFORE DEPLYMENT!!!!!
			printf(".");
		}
		if (timer1min%5==0){
			//measure sun
			suntmp=sun();
			if (suntmp > maxsun){
			  maxsun=suntmp;
			}
		}
		if (timer1min==60){
			timer1h++;
			hours++;
			timer1min=0;
			//Printout of status, maybe remove
			printf("\n\rWater: %d, Sun: %d, Max sun:%d, day no:%d, Time: %d :00 \n\r", water(), sun(), maxsun, days, hours);
			
		}
		if (hours==24){
			printf("Hours=24, updating days (%d)n\r",days);
			days++;
			hours=0;
			
		}
	}
}

//There is a LED that can blink on RB5, not used now
void blinkLed(void){
	unsigned char i=0;
	for (i=0;i<10;i++){
		RB5=1;	
		delay10ms(3);
		RB5=0;
		delay10ms(3);
    }
}


static void interrupt isr(void){
	if(ULPWUIF){
    	ULPWUE=0;
    	ULPWUIF=0;
  	}
}

void log(unsigned char days, unsigned char hours, unsigned char maxsun, unsigned char water){  //logs and prints
	unsigned char i=0;
	unsigned char tmp;
	unsigned char index=0;
	//implement as circular buffer
	//address 0 holds the current index
	//3 bytes/day 

	if(water >=250){
	  hours=hours+0b10000000;  	//UGLY, highest bit in hours is if watering was performed this day
	}
	
	index=eeprom_read(0);
	index=index+3;
	if (index>249){
	  index=2;  //UGLY, but this will be correct since the index happened to start at 255 (My EEPROM was 0xFF on every location)
	}
	eeprom_write(0,index);
	eeprom_write(index,days);
	eeprom_write(index+1,hours);
	eeprom_write(index+2,maxsun);
	
	//print the log
	printf("Logprint: \n\r");
	printf("Current index (after update) is %d\n\r",eeprom_read(0));
	
	//REMOVE if-statement BEFORE DEPLOYMENT!!!!!! (used for debugging
	//if(days%5==4){
	
	for(i=1;i<255;i++){
	  //day (0-80?)
	  //water sensor wet/dry
	  //timer1h
	  printf("Byte %03d: ",i);
	  tmp=eeprom_read(i);
	  printf("%d ",tmp);
	  if(i%3==0){
		printf("(Time/water, Time=%d, Water=%d 1:yes, 0:no)\n\r", (tmp&0b01111111), ((tmp&0b10000000)>>7));
	  }
	  if(i%3==1){
		printf("(Max sun)\n\r");
	  }
	  if(i%3==2){
		printf("(Day no)\n\r");
	  }
	}
	//REMOVE BELOW BEFORE DEPLOYMENT!!!!!!
	//}else{
	//	printf("Skipping logprint\n\r");
	//}
	//REMOVE ABOVE BEFORE DEPLOYMENT!!!!!!
	
	printf("Logprint done!\n\r");
}

unsigned char water(){
  unsigned char ret=0;
  
  //Supply power for water sensor voltage divider
  TRISA1=OUT;
  RA1=1;
  delay10ms(1);
  
  //read the analog pin connected to the watersensor, use 8-bit
  ret=(unsigned char) (readAd(WATER)>>2);
  RA1=0; 
  TRISA1=IN;
  return ret;
}

unsigned char sun(){
  unsigned char ret=0;
  //Supply ground for solar panel voltage divider
  TRISC1=OUT;
  RC1=0;
  delay10ms(1);
  
  //read the analog pin connected to the solar panel, use 8-bit
  ret=(unsigned char) (readAd(SUN)>>2);
  RC1=0;
  TRISC1=IN;  
  return ret;
}

void initAd(){
	ADCS1=0;
	ADCS0=1;		
	VCFG1=0;		//Vss is used as voltage reference
	VCFG0=0;		//Vdd is used as voltage reference
	//ADON=1;
}

unsigned short int readAd(unsigned char channel){
	unsigned short int ret=0;
  if(channel==SUN){  //sun sensor
    CHS3=0;
    CHS2=0;
    CHS1=1;
    CHS0=1;
  }else if(channel==WATER){  //water sensor
    CHS3=0;
    CHS2=0;
    CHS1=1;
    CHS0=0;
  }
	ADFM=1;				//8 least significant bits in ADRESL
	ADON=1;
	//delay100us(1000);		//set-up  time for A/D module
	GO=1;					//start conversion
	while (GO){		//wait for conversion to complete
		asm("NOP");
	}
	ret=(ADRESH<<8) + ADRESL;
	ADON=0;	
	return ret;
}

void initEusart(void){

	//set rx as input, tx as output
	TRISB7=0;
	
	//init TX module, set up baud-rate and enable TX module to send chars to serial line. See chapter 12.1.1.6 on page 152 in data-sheet
	TXIE=0;						//disable interrupts on eusart TX module
	
	BRGH = 1;					//baud-rate 1200 for project irrigation, see table 12-5 in datasheet page 163
	BRG16 = 1;				
	SPBRG=5;			
	SPBRGH=0;		
	
	SYNC = 0;					//asyncronous mode
	SPEN=1;						//enable serial communication?
	TXEN=1;						//enable tX module
	
	//init RX module
	CREN=1;						//enable RX module
	RCIE=0;						//disable interrupts on RX when char is received (they are not handled)
}

void putch(unsigned char c)		//required for library function printf
{
	asm("NOP");
	asm("NOP");
	asm("NOP");
	while(!TXIF){  				//wait until tx-buffer is empty
		asm("NOP");
	}
	TXREG=c;					//put char in TX buffer to start the transmission.
}

	
void delay10ms(unsigned short int ms){  //This is probably not 10ms...
	unsigned short int i,j,k;
	for(k=0;k<ms;k++){
		asm("NOP");
    asm("NOP");
    asm("NOP");
    asm("NOP");
    asm("NOP");
    asm("NOP");
    asm("NOP");
    asm("NOP");
    asm("NOP");
    asm("NOP");
    for(i=0;i<3;i++){
			asm("NOP");
		}
	}
}

